package come.stormborntales.notevault

import android.net.Uri
import android.os.Bundle
import android.webkit.MimeTypeMap
import androidx.activity.ComponentActivity
import androidx.activity.compose.rememberLauncherForActivityResult
import androidx.activity.compose.setContent
import androidx.activity.result.PickVisualMediaRequest
import androidx.activity.result.contract.ActivityResultContracts
import androidx.compose.runtime.*
import androidx.compose.ui.platform.LocalContext
import androidx.lifecycle.viewmodel.compose.viewModel
import come.stormborntales.notevault.data.local.AppDatabase
import come.stormborntales.notevault.data.model.NoteEntry
import come.stormborntales.notevault.data.repository.NoteRepository
import come.stormborntales.notevault.ui.screens.AddNoteDialog
import come.stormborntales.notevault.ui.screens.MainScreen
import come.stormborntales.notevault.ui.theme.NoteVaultTheme
import come.stormborntales.notevault.ui.viewmodel.NoteViewModel
import come.stormborntales.notevault.ui.viewmodel.NoteViewModelFactory
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import java.io.File
import java.io.InputStream

class MainActivity : ComponentActivity() {
    override fun onCreate(savedInstanceState: Bundle?) {
        super.onCreate(savedInstanceState)
        val applicationContext = applicationContext
        val database = AppDatabase.getDatabase(applicationContext)
        val repository = NoteRepository(database.noteDao())
        val viewModelFactory = NoteViewModelFactory(repository)

        setContent {
            NoteVaultTheme {
                val viewModel: NoteViewModel = viewModel(factory = viewModelFactory)
                val context = LocalContext.current

                // Globale Notenliste
                val notes = remember { mutableStateListOf<NoteEntry>() }

                // Bildauswahl + Dialog-States
                var selectedUris by remember { mutableStateOf<List<Uri>>(emptyList()) }
                var showDialog by remember { mutableStateOf(false) }

                // Launcher innerhalb von Compose
                val imagePickerLauncher = rememberLauncherForActivityResult(
                    contract = ActivityResultContracts.OpenMultipleDocuments(),
                    onResult = { uris ->
                        if (uris.isNotEmpty()) {
                            selectedUris = uris
                            showDialog = true
                        }
                    }
                )

                // UI anzeigen
                MainScreen(
                    viewModel = viewModel,
                    onAddNoteClicked = {
                        imagePickerLauncher.launch(
                            arrayOf("image/*")
                        )
                    }
                )

                // Dialog bei Bedarf
                if (showDialog) {
                    val context = LocalContext.current;
                    val scope = rememberCoroutineScope();
                    AddNoteDialog(
                        onDismiss = { showDialog = false },
                        onSave = { title, composer, year, genre, description ->
                            viewModel.addNote(
                                context = context,
                                title = title,
                                composer = composer,
                                year = year,
                                genre = genre,
                                description = description,
                                selectedUris = selectedUris,
                                onDone = { showDialog = false }
                            )
                        }
                    )
                }
            }
        }
    }
}
